from __future__ import absolute_import

from .data_generator import DataGenerator
from .data_generator_labels import DataGeneratorWithLabels
from .data_loader import DataLoader
from .incremental_beta_callback import IncrementalBeta


from .ladder_vae import LadderVAE
from keras.layers import Input, Dense, ReLU, BatchNormalization
from keras.models import Model


class SmallDenseLadderVAE_BN(LadderVAE):
    def encoder1(self):
        input_tensor = Input(shape=self.input_shape[1:])
        x = Dense(512)(input_tensor)
        x = BatchNormalization()(x)
        x = ReLU()(x)
        x = Dense(256)(x)
        x = BatchNormalization()(x)
        x = ReLU()(x)
        x = Dense(self.latent_dim1)(x)
        encoder = Model(input_tensor, x, name="small_dense_encoder_1")
        return encoder

    def encoder2(self):
        input_tensor = Input(shape=(self.latent_dim1,))
        x = Dense(256)(input_tensor)
        x = BatchNormalization()(x)
        x = ReLU()(x)
        x = Dense(128)(x)
        x = BatchNormalization()(x)
        x = ReLU()(x)
        encoder = Model(input_tensor, x, name="small_dense_encoder_2")
        return encoder

    def decoder2(self):
        latent2 = Input(shape=(self.latent_dim2,))
        x = Dense(256)(latent2)
        x = BatchNormalization()(x)
        x = ReLU()(x)
        x = Dense(self._mean_variance_input_shape)(x)
        x = BatchNormalization()(x)
        x = ReLU()(x)
        decoder = Model(latent2, x, name="small_dense_decoder_2")
        return decoder

    def decoder1(self):
        latent1 = Input(shape=(self.latent_dim1,))
        x = Dense(256)(latent1)
        x = BatchNormalization()(x)
        x = ReLU()(x)
        x = Dense(512)(x)
        x = BatchNormalization()(x)
        x = ReLU()(x)
        x = Dense(self.input_shape[1])(x)
        decoder = Model(latent1, x, name="small_dense_decoder_1")
        return decoder



from keras.callbacks import Callback
import numpy as np
import matplotlib.pyplot as plt
import os
import imageio


class GifCallBack(Callback):
    def __init__(self, datagen, generator, latent_dim, grid_size=4):
        self.datagen = datagen
        self.generator = generator
        self.grid_size = grid_size
        self.latent_dim = latent_dim
        self.latent_inputs = np.random.normal(
            size=(grid_size*grid_size, latent_dim))

    def on_epoch_end(self, epoch, logs=None):
        self._make_gif(epoch)

    def on_train_end(self, logs=None):
        images = []
        for filename in os.listdir(os.getcwd() + '/tmp'):
            images.append(imageio.imread(os.getcwd() + '/tmp/' + filename))
            os.remove(os.getcwd() + '/tmp/' + filename)
        imageio.mimsave(os.getcwd() + '/latent.gif', images)
        os.rmdir(os.getcwd() + '/tmp')

    def _make_gif(self, epoch):
        recos = self._plot_samples(self.latent_inputs)

        fig, axes = plt.subplots(
            self.grid_size, self.grid_size, sharex=True, sharey=True, figsize=(11, 11))

        for ind, ax in enumerate(axes.flatten()):
            ax.imshow(recos[ind], interpolation='none')
            ax.set_xticks([])
            ax.set_yticks([])

        fig.savefig(os.getcwd() + "/tmp/" + str(epoch) + ".png", dpi=100)
        plt.close(fig)

    def _plot_samples(self, latent_inputs):
        recos = self.generator.predict(
            latent_inputs[:self.grid_size*self.grid_size].reshape(self.grid_size**2, self.latent_dim))

        if np.prod(recos[0].shape) / (28*28) != 1:
            recos = recos.reshape(
                self.grid_size**2, 28, 28, int(np.prod(recos[0].shape) / (28*28)))
        else:
            recos = recos.reshape(self.grid_size**2, 28, 28)

        return recos

    def _make_on_train_start(self):
        os.mkdir(os.getcwd() + '/tmp')

    def _remove_on_error(self):
        for filename in os.listdir(os.getcwd() + '/tmp'):
            os.remove(os.getcwd() + '/tmp/' + filename)
        os.rmdir(os.getcwd() + '/tmp')



from .conv_lvae import ConvLadderVAE
from .dense_lvae import DenseLadderVAE
from .small_dense_lvae import SmallDenseLadderVAE
from .small_dense_lvae_bn import SmallDenseLadderVAE_BN
from .linear_encoder_lvae import SmallDenseLinearLadderVAE



from keras.preprocessing.image import ImageDataGenerator
import numpy as np
from .data_loader import DataLoader


class DataGeneratorWithLabels:
    def __init__(self,
                 image_shape,
                 batch_size,
                 file_path='/data/scramtex_700_28px.pkl',
                 binarize=False,
                 whiten=False):
        self.DATA_LOADER = DataLoader(file_path, binarize, whiten)
        self.IMAGE_SHAPE = image_shape
        self.BATCH_SIZE = batch_size
        self.DATA_GENERATOR = ImageDataGenerator()
        self.TRAIN, self.TRAIN_LABEL, _, _ = self.DATA_LOADER.get_data_and_labels(
        )
        print("Train SHAPE : ", self.TRAIN.shape)

    def flow(self):
        return self.DATA_GENERATOR.flow(self.TRAIN,
                                        self.TRAIN_LABEL,
                                        batch_size=self.BATCH_SIZE)
